
// The D/Objective-C Bridge
// Copyright © 2007  Michel Fortin
// <http://michelf.com/d-objc-bridge/>
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.

module cocoa.appkit.menu;

import cocoa.appkit.menuitem;

import cocoa.foundation.object;
import cocoa.foundation.array;
import cocoa.foundation.string;
import cocoa.foundation.geometry;

import objc.bridge;
import objc.runtime;


class NSMenu : NSObject {
	mixin ObjcWrap;
	this(objc.runtime.id self) { super(self); }
	

	/** Create menu with title. */
	this(NSString title) {
		super(objcAlloc);
		objcInstance = invokeObjcSuper!(objc.runtime.id, "initWithTitle:")(title);
	}
	/** ditto */
	this(char[] title) {
		this(title.toNSString());
	}
	
	
	/** Insert item at index. */
	NSMenuItem insertItem(NSMenuItem item, int index) {
		return invokeObjcSuper!(NSMenuItem, "insertItem:atIndex:")(item, index);
	}
	
	/** ditto */
	NSMenuItem insertItem(NSString title, objc.runtime.SEL action, NSString keyEquivalent, int index) {
		return invokeObjcSuper!(NSMenuItem, "insertItemWithTitle:action:keyEquivalent:atIndex:")(title, action, keyEquivalent, index);
	}
	
	/** ditto */
	final NSMenuItem insertItem(char[] title, objc.runtime.SEL action, char[] keyEquivalent, int index) {
		return insertItem(title.toNSString(), action, keyEquivalent.toNSString(), index);
	}
	
	
	/** Add item at the end of menu. */
	NSMenuItem addItem(NSMenuItem item) {
		return invokeObjcSuper!(NSMenuItem, "addItem:")(item);
	}
	
	/** ditto */
	NSMenuItem addItem(NSString title, objc.runtime.SEL action, NSString keyEquivalent) {
		return invokeObjcSuper!(NSMenuItem, "addItemWithTitle:action:keyEquivalent:")(title, action, keyEquivalent);
	}
	
	/** ditto */
	final NSMenuItem addItem(char[] title, objc.runtime.SEL action, char[] keyEquivalent) {
		return addItem(title.toNSString(), action, keyEquivalent.toNSString());
	}
	
	
	/** Remove item at index. */
	void removeItem(int index) {
		invokeObjcSuper!(void, "removeItemAtIndex:")(index);
	}
	
	/** Remove item. */
	void removeItem(NSMenuItem item) {
		invokeObjcSuper!(void, "removeItem:")(item);
	}
	
	
	/** Set submenu for item. */
	void setSubmenu(NSMenu submenu, NSMenuItem item) {
		invokeObjcSuper!(void, "setSubmenu:forItem:")(submenu, item);
	}
	

	mixin ObjcBindMethod!(insertItem, NSMenuItem, "insertItem:atIndex:", NSMenuItem, int);
	mixin ObjcBindMethod!(insertItem, NSMenuItem, "insertItemWithTitle:action:keyEquivalent:atIndex:", NSString, objc.runtime.SEL, NSString, int);
	
	mixin ObjcBindMethod!(addItem, NSMenuItem, "addItem:", NSMenuItem);
	mixin ObjcBindMethod!(addItem, NSMenuItem, "addItemWithTitle:action:keyEquivalent:", NSString, objc.runtime.SEL, NSString);
	
	mixin ObjcBindMethod!(removeItem, void, "removeItemAtIndex:", int);
	mixin ObjcBindMethod!(removeItem, void, "removeItem:", NSMenuItem);
	mixin ObjcBindMethod!(setSubmenu, void, "setSubmenu:forItem:", NSMenu, NSMenuItem);
	
	
	/** Array of all the items in the menu. */
	NSArray items() {
		return invokeObjcSuper!(NSArray, "itemArray")();
	}
	
	/** Number of items in the menu. */
	int numberOfItems() {
		return invokeObjcSuper!(int, "numberOfItems")();
	}

	mixin ObjcBindMethod!(items, NSArray, "itemArray");
	mixin ObjcBindMethod!(numberOfItems, int, "numberOfItems");
	
	
	int indexOfItem(NSMenuItem item) {
		return invokeObjcSuper!(int, "indexOfItem:")(item);
	}
	
	int indexOfItem(NSString title) {
		return invokeObjcSuper!(int, "indexOfItemWithTitle:")(title);
	}
	
	int indexOfItemWithTag(uint tag) {
		return invokeObjcSuper!(int, "indexOfItemWithTag:")(tag);
	}
	
	int indexOfItemWithRepresentedObject(Object o) {
		return invokeObjcSuper!(int, "indexOfItemWithRepresentedObject:")(o);
	}
	
	int indexOfItemWithSubmenu(NSMenu submenu) {
		return invokeObjcSuper!(int, "indexOfItemWithSubmenu:")(submenu);
	}
	
	int indexOfItemWithTargetAndAction(Object target, SEL action) {
		return invokeObjcSuper!(int, "indexOfItemWithTarget:andAction:")(target, action);
	}
	
	mixin ObjcBindMethod!(indexOfItem, int, "indexOfItem:", NSMenuItem);
	mixin ObjcBindMethod!(indexOfItem, int, "indexOfItemWithTitle:", NSString);
	mixin ObjcBindMethod!(indexOfItemWithTag, int, "indexOfItemWithTag:", int);
	mixin ObjcBindMethod!(indexOfItemWithRepresentedObject, int, "indexOfItemWithRepresentedObject:", Object);
	mixin ObjcBindMethod!(indexOfItemWithSubmenu, int, "indexOfItemWithSubmenu:", NSMenu);
	mixin ObjcBindMethod!(indexOfItemWithTargetAndAction, int, "indexOfItemWithTarget:andAction:", Object, objc.runtime.SEL);
	
	
	
	NSMenuItem opIndex(int index) {
		return invokeObjcSuper!(NSMenuItem, "itemAtIndex:")(index);
	}
	
	NSMenuItem opIndex(NSString title) {
		return invokeObjcSuper!(NSMenuItem, "itemWithTitle:")(title);
	}
	
	final NSMenuItem opIndex(char[] title) {
		return opIndex(title.toNSString());
	}
	
	NSMenuItem itemWithTag(int tag) {
		return invokeObjcSuper!(NSMenuItem, "itemWithTag:")(tag);
	}

	mixin ObjcBindMethod!(opIndex, NSMenuItem, "itemAtIndex:", int);
	mixin ObjcBindMethod!(opIndex, NSMenuItem, "itemWithTitle:", NSString);
	mixin ObjcBindMethod!(itemWithTag, NSMenuItem, "itemWithTag:", int);
	
	
	void autoenablesItems(bool flag) {
		invokeObjcSuper!(void, "setAutoenablesItems:")(flag);
	}
	bool autoenablesItems() {
		return invokeObjcSuper!(bool, "autoenablesItems")();
	}
	
	mixin ObjcBindMethod!(autoenablesItems, void, "setAutoenablesItems:", bool);
	mixin ObjcBindMethod!(autoenablesItems, bool, "autoenablesItems");
	
	
	static void menuBarVisible(bool visible) {
		invokeObjcSuperClass!(void, "setMenuBarVisible:")(visible);
	}
	
	static bool menuBarVisible() {
		return invokeObjcSuperClass!(bool, "menuBarVisible")();
	}

}



