
// The D/Objective-C Bridge
// Copyright © 2007  Michel Fortin
// <http://michelf.com/d-objc-bridge/>
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.

module cocoa.foundation.range;

private import objc.runtime;


extern (C) {

	NSRange NSUnionRange(NSRange range1, NSRange range2);
	NSRange NSIntersectionRange(NSRange range1, NSRange range2);
	objc.runtime.id NSStringFromRange(NSRange range);
	NSRange NSRangeFromString(objc.runtime.id aString);
	
}


struct NSRange {

    uint location;
    uint length;
	
	// Commodity function to work with d slices.
	alias location start;
	uint end() { return location + length; }
	void end(uint value) { length = end - location; }
	
	bool locationInRange(uint loc) { return loc - location < length; }
	
	static alias NSUnionRange rangeFromUnion;
	static alias NSIntersectionRange rangeFromIntersection;
	
	static NSRange opSlice(uint from, uint to)
	in {
		assert(to >= from);
	}
	body {
		NSRange range;
		range.location = from;
		range.length = to - from;
		return range;
	}
	
	NSRange unite(NSRange range) {
		return NSUnionRange(*this, range);
	}
	
	NSRange intersect(NSRange range) {
		return NSIntersectionRange(*this, range);
	}
		
//	NSString toNSString() { 
//		return NSString.objcWrap(NSStringFromRange(*this));
//	}
//	
//	char[] toString() {
//		return toNSString.toString;
//	}
//	
//	static NSRange fromString(NSString string) {
//		return NSRangeFromString(string.objcInstance);
//	}
//	
//	static NSRange fromString(char[] string) {
//		return NSRange.fromString(.toNSString(string));
//	}

}

unittest {

	NSRange range = NSRange[1..3];
	assert(range.location == 1);
	assert(range.length == 2);
	assert(range.end == 3);

}


alias NSRange* NSRangePointer;

static assert(NSRange.sizeof == 8);



//@interface NSValue (NSValueRangeExtensions)
//
//+ (NSValue *)valueWithRange:(NSRange)range;
//- (NSRange)rangeValue;
//
//@end

