
// The D/Objective-C Bridge
// Copyright © 2007  Michel Fortin
// <http://michelf.com/d-objc-bridge/>
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.

module objc.msg;

import objc.runtime;


/**
 * Return-type-casted message sending function with object id.
 */
template send(R) {
	const send = TObjcMsg!(R).send;
}

/**
 * Return-type-casted message sending function with object context.
 */
template sendSuper(R) {
	const sendSuper = TObjcMsg!(R).sendSuper;
}


// -- Implementation Templates

private:
 
// No return type.
template TObjcMsg(T : void) {
extern (C):
	const send = cast(void function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(void function(objc_super*, SEL, ...))&objc_msgSendSuper;
}

// Integral types.
template TObjcMsg(T : bool) {
extern (C):
	const send = cast(BOOL function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(BOOL function(objc_super*, SEL, ...))&objc_msgSendSuper;
}
template TObjcMsg(T : byte) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
}
template TObjcMsg(T : ubyte) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
}
template TObjcMsg(T : short) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
}
template TObjcMsg(T : ushort) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
}
template TObjcMsg(T : int) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
}
template TObjcMsg(T : uint) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
}
template TObjcMsg(T : long) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
}
template TObjcMsg(T : ulong) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
}
template TObjcMsg(T : char) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
}
template TObjcMsg(T : wchar) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
}
template TObjcMsg(T : dchar) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
}
template TObjcMsg(T : T*) {
extern (C):
	const send = cast(T* function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(T* function(objc_super*, SEL, ...))&objc_msgSendSuper;
}

// Objects variables in D are actually pointers. Let it work that way.
// Note that automatic type conversion makes this unlikely to be used.
template TObjcMsg(T : Object) {
	const send = cast(Object function(id, SEL, ...))&objc_msgSend;
	const sendSuper = cast(Object function(objc_super*, SEL, ...))&objc_msgSendSuper;
}

// Floating point types.
// Note: real floating point type may not have an equivalent in Objective-C.
version (X86) {
	// The runtime use a special message function for floating points on X86
	template TObjcMsg(T : float) {
	extern (C):
		const send = cast(T function(id, SEL, ...))&objc_msgSend_fpret;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper_fpret;
	}
	template TObjcMsg(T : double) {
	extern (C):
		const send = cast(T function(id, SEL, ...))&objc_msgSend_fpret;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper_fpret;
	}
	template TObjcMsg(T : real) {
	extern (C):
		const send = cast(T function(id, SEL, ...))&objc_msgSend_fpret;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper_fpret;
	}
	template TObjcMsg(T : ifloat) {
	extern (C):
		const send = cast(T function(id, SEL, ...))&objc_msgSend_fpret;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper_fpret;
	}
	template TObjcMsg(T : idouble) {
	extern (C):
		const send = cast(T function(id, SEL, ...))&objc_msgSend_fpret;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper_fpret;
	}
	template TObjcMsg(T : ireal) {
	extern (C):
		const send = cast(T function(id, SEL, ...))&objc_msgSend_fpret;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper_fpret;
	}
} else {
	template TObjcMsg(T : float) {
	extern (C):
		const send = cast(T function(id, SEL, ...))&objc_msgSend;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
	}
	template TObjcMsg(T : double) {
	extern (C):
		const send = cast(T function(id, SEL, ...))&objc_msgSend;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
	}
	template TObjcMsg(T : real) {
	extern (C):
		const send = cast(T function(id, SEL, ...))&objc_msgSend;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
	}
	template TObjcMsg(T : ifloat) {
	extern (C):
		const send = cast(T function(id, SEL, ...))&objc_msgSend;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
	}
	template TObjcMsg(T : idouble) {
	extern (C):
		const send = cast(T function(id, SEL, ...))&objc_msgSend;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
	}
	template TObjcMsg(T : ireal) {
	extern (C):
		const send = cast(T function(id, SEL, ...))&objc_msgSend;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
	}
}

// Complex numbers, passed as struct { re, im }.
template TObjcMsg(T : cfloat) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend_stret;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper_stret;
}
template TObjcMsg(T : cdouble) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend_stret;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper_stret;
}
template TObjcMsg(T : creal) {
extern (C):
	const send = cast(T function(id, SEL, ...))&objc_msgSend_stret;
	const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper_stret;
}


// Other types.
template TObjcMsg(T) {
extern (C):
	static if (is(T == struct) || is(T == union)) {
		const send = cast(T function(id, SEL, ...))&objc_msgSend_stret;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper_stret;
	}
	else static if (is(T == enum)) {
		const send = cast(T function(id, SEL, ...))&objc_msgSend;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
	}
	else static if (T.mangleof[0] == 'G') {
		// Static array. Returned as a pointer?
		const send = cast(typeof(T.ptr) function(id, SEL, ...))&objc_msgSend;
		const sendSuper = cast(typeof(T.ptr) function(objc_super*, SEL, ...))&objc_msgSendSuper;
	}
	else static if (T.mangleof[0] == 'A') {
		// Dynamic array. Passed as struct.
		const send = cast(T function(id, SEL, ...))&objc_msgSend_stret;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper_stret;
	}
	else {
		const send = cast(T function(id, SEL, ...))&objc_msgSend;
		const sendSuper = cast(T function(objc_super*, SEL, ...))&objc_msgSendSuper;
	}
}



unittest {
	// Making sure the right function is used for each type.
	// Integral types
	assert (TObjcMsg!(bool).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(byte).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(ubyte).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(short).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(ushort).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(int).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(uint).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(long).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(ulong).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(char).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(wchar).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(dchar).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(Object).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(void*).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(bit*).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(char*).send is cast(void*)&objc_msgSend);
	// Floating point
	version (X86) {
		assert (TObjcMsg!(float).send is cast(void*)&objc_msgSend_fpret);
		assert (TObjcMsg!(double).send is cast(void*)&objc_msgSend_fpret);
		assert (TObjcMsg!(real).send is cast(void*)&objc_msgSend_fpret);
	} else {
		assert (TObjcMsg!(float).send is cast(void*)&objc_msgSend);
		assert (TObjcMsg!(double).send is cast(void*)&objc_msgSend);
		assert (TObjcMsg!(real).send is cast(void*)&objc_msgSend);
	}
	// Complex, passed as struct
	assert (TObjcMsg!(cfloat).send is cast(void*)&objc_msgSend_stret);
	assert (TObjcMsg!(cdouble).send is cast(void*)&objc_msgSend_stret);
	assert (TObjcMsg!(creal).send is cast(void*)&objc_msgSend_stret);
	// Array, passed as struct
	// Struct
	struct StructType { int a, b, c, d, e; }
	assert (TObjcMsg!(StructType).send is cast(void*)&objc_msgSend_stret);
	// Pointer
	assert (TObjcMsg!(StructType*).send is cast(void*)&objc_msgSend);
	// Typedef
	typedef long LongTypedef;
	typedef Object ObjectTypedef;
	typedef cfloat CFloatTypedef;
	assert (TObjcMsg!(LongTypedef).send is cast(void*)&objc_msgSend);
	assert (TObjcMsg!(ObjectTypedef).send is cast(void*)&objc_msgSend);
	version (X86) {
		assert (TObjcMsg!(CFloatTypedef).send is cast(void*)&objc_msgSend_fpret);
	} else {
		assert (TObjcMsg!(CFloatTypedef).send is cast(void*)&objc_msgSend);
	}
	// Enum
	enum BitEnum : bit { A, B }
	enum ByteEnum : byte { A, B, C }
	enum IntEnum : int { A, B, C, D }
	enum LongEnum : long { A, B, C, D, E }
	assert(TObjcMsg!(BitEnum).send is cast(void*)&objc_msgSend);
	assert(TObjcMsg!(ByteEnum).send is cast(void*)&objc_msgSend);
	assert(TObjcMsg!(IntEnum).send is cast(void*)&objc_msgSend);
	assert(TObjcMsg!(LongEnum).send is cast(void*)&objc_msgSend);
}
